"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3898],{93898:function(e){e.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"141414","token":""},{"foreground":"5f5a60","fontStyle":"italic","token":"comment"},{"foreground":"cf6a4c","token":"constant"},{"foreground":"9b703f","token":"entity"},{"foreground":"cda869","token":"keyword"},{"foreground":"f9ee98","token":"storage"},{"foreground":"8f9d6a","token":"string"},{"foreground":"9b859d","token":"support"},{"foreground":"7587a6","token":"variable"},{"foreground":"d2a8a1","fontStyle":"italic underline","token":"invalid.deprecated"},{"foreground":"f8f8f8","background":"562d56bf","token":"invalid.illegal"},{"background":"b0b3ba14","token":"text source"},{"background":"b1b3ba21","token":"text.html.ruby source"},{"foreground":"9b5c2e","fontStyle":"italic","token":"entity.other.inherited-class"},{"foreground":"daefa3","token":"string source"},{"foreground":"ddf2a4","token":"string constant"},{"foreground":"e9c062","token":"string.regexp"},{"foreground":"cf7d34","token":"string.regexp constant.character.escape"},{"foreground":"cf7d34","token":"string.regexp source.ruby.embedded"},{"foreground":"cf7d34","token":"string.regexp string.regexp.arbitrary-repitition"},{"foreground":"8a9a95","token":"string variable"},{"foreground":"dad085","token":"support.function"},{"foreground":"cf6a4c","token":"support.constant"},{"foreground":"8996a8","token":"meta.preprocessor.c"},{"foreground":"afc4db","token":"meta.preprocessor.c keyword"},{"foreground":"494949","token":"meta.tag.sgml.doctype"},{"foreground":"494949","token":"meta.tag.sgml.doctype entity"},{"foreground":"494949","token":"meta.tag.sgml.doctype string"},{"foreground":"494949","token":"meta.tag.preprocessor.xml"},{"foreground":"494949","token":"meta.tag.preprocessor.xml entity"},{"foreground":"494949","token":"meta.tag.preprocessor.xml string"},{"foreground":"ac885b","token":"declaration.tag"},{"foreground":"ac885b","token":"declaration.tag entity"},{"foreground":"ac885b","token":"meta.tag"},{"foreground":"ac885b","token":"meta.tag entity"},{"foreground":"e0c589","token":"declaration.tag.inline"},{"foreground":"e0c589","token":"declaration.tag.inline entity"},{"foreground":"e0c589","token":"source entity.name.tag"},{"foreground":"e0c589","token":"source entity.other.attribute-name"},{"foreground":"e0c589","token":"meta.tag.inline"},{"foreground":"e0c589","token":"meta.tag.inline entity"},{"foreground":"cda869","token":"meta.selector.css entity.name.tag"},{"foreground":"8f9d6a","token":"meta.selector.css entity.other.attribute-name.tag.pseudo-class"},{"foreground":"8b98ab","token":"meta.selector.css entity.other.attribute-name.id"},{"foreground":"9b703f","token":"meta.selector.css entity.other.attribute-name.class"},{"foreground":"c5af75","token":"support.type.property-name.css"},{"foreground":"f9ee98","token":"meta.property-group support.constant.property-value.css"},{"foreground":"f9ee98","token":"meta.property-value support.constant.property-value.css"},{"foreground":"8693a5","token":"meta.preprocessor.at-rule keyword.control.at-rule"},{"foreground":"ca7840","token":"meta.property-value support.constant.named-color.css"},{"foreground":"ca7840","token":"meta.property-value constant"},{"foreground":"8f9d6a","token":"meta.constructor.argument.css"},{"foreground":"f8f8f8","background":"0e2231","fontStyle":"italic","token":"meta.diff"},{"foreground":"f8f8f8","background":"0e2231","fontStyle":"italic","token":"meta.diff.header"},{"foreground":"f8f8f8","background":"0e2231","fontStyle":"italic","token":"meta.separator"},{"foreground":"f8f8f8","background":"420e09","token":"markup.deleted"},{"foreground":"f8f8f8","background":"4a410d","token":"markup.changed"},{"foreground":"f8f8f8","background":"253b22","token":"markup.inserted"},{"foreground":"f9ee98","token":"markup.list"},{"foreground":"cf6a4c","token":"markup.heading"}],"colors":{"editor.foreground":"#F8F8F8","editor.background":"#141414","editor.selectionBackground":"#DDF0FF33","editor.lineHighlightBackground":"#FFFFFF08","editorCursor.foreground":"#A7A7A7","editorWhitespace.foreground":"#FFFFFF40"}}')}}]);