(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6308],{29237:function(n,e,t){"use strict";var i=t(9518),r=t(67971),d=t(86673),o=t(19711),c=t(23831),a=t(73942),l=t(37391),u=t(49125),h=t(28598),s=(0,i.css)(["padding:","px;padding-bottom:","px;padding-top:","px;"],2*u.iI,1.5*u.iI,1.5*u.iI),f=i.default.div.withConfig({displayName:"Panel__PanelStyle",componentId:"sc-1ct8cgl-0"})(["border-radius:","px;overflow:hidden;"," "," "," "," "," "," "," "," "," ",""],a.n_,(function(n){return n.fullWidth&&"\n    width: 100%;\n  "}),(function(n){return"\n    border: 1px solid ".concat((n.theme.interactive||c.Z.interactive).defaultBorder,";\n  ")}),(function(n){return!n.dark&&"\n    background-color: ".concat((n.theme.background||c.Z.background).panel,";\n  ")}),(function(n){return n.dark&&"\n    background-color: ".concat((n.theme.background||c.Z.background).content,";\n  ")}),(function(n){return!n.fullHeight&&"\n    height: fit-content;\n  "}),(function(n){return n.maxHeight&&"\n    max-height: ".concat(n.maxHeight,";\n  ")}),(function(n){return n.maxWidth&&"\n    max-width: ".concat(n.maxWidth,"px;\n  ")}),(function(n){return n.minWidth&&"\n    min-width: ".concat(n.minWidth,"px;\n\n    @media (max-width: ").concat(n.minWidth,"px) {\n      min-width: 0;\n    }\n  ")}),(function(n){return n.borderless&&"\n    border: none;\n  "}),(function(n){return n.overflowVisible&&"\n    overflow: visible;\n  "})),g=i.default.div.withConfig({displayName:"Panel__HeaderStyle",componentId:"sc-1ct8cgl-1"})(["border-top-left-radius:","px;border-top-right-radius:","px;"," "," "," ",""],a.n_,a.n_,(function(n){return"\n    background-color: ".concat((n.theme.background||c.Z.background).chartBlock,";\n    border-bottom: 1px solid ").concat((n.theme.interactive||c.Z.interactive).defaultBorder,";\n  ")}),(function(n){return n.height&&"\n    height: ".concat(n.height,"px;\n  ")}),s,(function(n){return n.headerPaddingVertical&&"\n    padding-bottom: ".concat(n.headerPaddingVertical,"px;\n    padding-top: ").concat(n.headerPaddingVertical,"px;\n  ")})),p=i.default.div.withConfig({displayName:"Panel__ContentStyle",componentId:"sc-1ct8cgl-2"})(["overflow-y:auto;padding:","px;height:100%;"," "," "," "," ",""],1.75*u.iI,l.w5,(function(n){return n.height&&"\n    height: ".concat(n.height,"px;\n  ")}),(function(n){return n.maxHeight&&"\n    max-height: calc(".concat(n.maxHeight," - ").concat(15*u.iI,"px);\n  ")}),(function(n){return n.noPadding&&"\n    padding: 0;\n  "}),(function(n){return n.overflowVisible&&"\n    overflow: visible;\n  "})),x=i.default.div.withConfig({displayName:"Panel__FooterStyle",componentId:"sc-1ct8cgl-3"})(["border-style:",";border-top-width:","px;padding:","px;"],a.M8,a.YF,1.75*u.iI);e.Z=function(n){var e=n.borderless,t=n.children,i=n.containerRef,c=n.contentContainerRef,a=n.dark,l=n.footer,u=n.fullHeight,s=void 0===u||u,m=n.fullWidth,b=void 0===m||m,v=n.header,w=n.headerHeight,_=n.headerIcon,k=n.headerPaddingVertical,P=n.headerTitle,Z=n.maxHeight,j=n.maxWidth,I=n.minWidth,W=n.noPadding,H=n.overflowVisible,y=n.subtitle;return(0,h.jsxs)(f,{borderless:e,dark:a,fullHeight:s,fullWidth:b,maxHeight:Z,maxWidth:j,minWidth:I,overflowVisible:H,ref:i,children:[(v||P)&&(0,h.jsxs)(g,{headerPaddingVertical:k,height:w,children:[v&&v,P&&(0,h.jsx)(r.Z,{alignItems:"center",justifyContent:"space-between",children:(0,h.jsxs)(r.Z,{alignItems:"center",children:[_&&_,(0,h.jsx)(d.Z,{ml:_?1:0,children:(0,h.jsx)(o.ZP,{bold:!0,default:!0,children:P})})]})})]}),(0,h.jsxs)(p,{maxHeight:Z,noPadding:W,overflowVisible:H,ref:c,children:[y&&(0,h.jsx)(d.Z,{mb:2,children:(0,h.jsx)(o.ZP,{default:!0,children:y})}),t]}),l&&(0,h.jsx)(x,{children:l})]})}},96283:function(n,e,t){"use strict";t.r(e);var i=t(77837),r=t(38860),d=t.n(r),o=t(29237),c=t(41788),a=t(30775),l=t(86673),u=t(20582),h=t(82531),s=t(49125),f=t(79585),g=t(9736),p=t(28598);function x(){var n=((0,g.PR)()||{}).id,e=h.ZP.users.detail(n).data,t=null===e||void 0===e?void 0:e.user;return(0,p.jsx)(a.Z,{uuidItemSelected:f.DQ,uuidWorkspaceSelected:f.tC,children:(0,p.jsx)(l.Z,{p:s.cd,children:(0,p.jsx)(o.Z,{children:(0,p.jsx)(u.Z,{disabledFields:["roles"],user:t})})})})}x.getInitialProps=(0,i.Z)(d().mark((function n(){return d().wrap((function(n){for(;;)switch(n.prev=n.next){case 0:return n.abrupt("return",{});case 1:case"end":return n.stop()}}),n)}))),e.default=(0,c.Z)(x)},10736:function(n,e,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/settings/account/profile",function(){return t(96283)}])}},function(n){n.O(0,[3850,2344,4506,5477,9774,2888,179],(function(){return e=10736,n(n.s=e);var e}));var e=n.O();_N_E=e}]);