(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5405],{63878:function(n,t,u){"use strict";u.r(t);var a=u(82684),i=u(34376),e=u(82531);t.default=function(){var n=(0,i.useRouter)(),t=n.asPath,u=t.split("?")[0],s=e.ZP.status.list().data;(0,a.useEffect)((function(){if(s){var a,i=null===s||void 0===s||null===(a=s.status)||void 0===a?void 0:a.is_instance_manager,e=t;"/"===u&&(e=i?"/manage":"/pipelines"),n.replace(e)}}),[s])}},48312:function(n,t,u){(window.__NEXT_P=window.__NEXT_P||[]).push(["/",function(){return u(63878)}])}},function(n){n.O(0,[9774,2888,179],(function(){return t=48312,n(n.s=t);var t}));var t=n.O();_N_E=t}]);