"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6042],{86042:function(e){e.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"161616","token":""},{"foreground":"ad2ea4","fontStyle":"italic","token":"comment"},{"foreground":"fc6f09","token":"keyword"},{"foreground":"fc6f09","token":"storage"},{"foreground":"fc83ff","token":"entity.other.inherited-class"},{"foreground":"58c554","token":"constant.numeric"},{"foreground":"1edafb","token":"constant"},{"foreground":"8dff0a","token":"constant.library"},{"foreground":"fc6f09","token":"support.function"},{"foreground":"fdc251","token":"constant.language"},{"foreground":"8dff0a","token":"string"},{"foreground":"1edafb","token":"support.type"},{"foreground":"8dff0a","token":"support.constant"},{"foreground":"fc6f09","token":"meta.tag"},{"foreground":"fc6f09","token":"declaration.tag"},{"foreground":"fc6f09","token":"entity.name.tag"},{"foreground":"ffff89","token":"entity.other.attribute-name"},{"foreground":"ffffff","background":"990000","token":"invalid"},{"foreground":"519f50","token":"constant.character.escaped"},{"foreground":"519f50","token":"constant.character.escape"},{"foreground":"519f50","token":"string source"},{"foreground":"519f50","token":"string source.ruby"},{"foreground":"e6e1dc","background":"144212","token":"markup.inserted"},{"foreground":"e6e1dc","background":"660000","token":"markup.deleted"},{"background":"2f33ab","token":"meta.diff.header"},{"background":"2f33ab","token":"meta.separator.diff"},{"background":"2f33ab","token":"meta.diff.index"},{"background":"2f33ab","token":"meta.diff.range"}],"colors":{"editor.foreground":"#E6E1DC","editor.background":"#161616","editor.selectionBackground":"#454545","editor.lineHighlightBackground":"#333435","editorCursor.foreground":"#FFFFFF","editorWhitespace.foreground":"#404040"}}')}}]);