DESTINATIONS = [
    dict(name='Amazon S3'),
    dict(name='BigQuery'),
    dict(name='Delta Lake S3'),
    dict(
        module_name='MSSQL',
        name='Microsoft SQL Server',
        uuid='mssql',
    ),
    dict(name='MySQL'),
    dict(name='PostgreSQL'),
    dict(name='Redshift'),
    dict(name='Snowflake'),
    dict(name='Trino'),
]
